import os
import numpy as np
import matplotlib.pyplot as plt
from tasks import itasks
from ase.collections import g2

data = {}

for task in itasks():
    #print(task.args)
    if not task.hasdir():
        continue

    with task.chdir():
        print('dir', os.getcwd())
        if os.path.exists('niter.txt'):
            with open('niter.txt') as fd:
                niter = int(fd.read())
        else:
            if not os.path.exists('status.txt'):
                niter = -1
            else:
                with open('status.txt') as fd:
                    status = fd.read().strip()

                if status != 'done':
                    niter = -1
                else:
                    niter = 0
                    with open('gpaw.txt') as fd:
                        for line in fd:
                            if line.startswith('Converged after '):
                                print('NITER', niter, line.strip())
                                niter += 1

            print('writing niter {}'.format(niter))
            with open('niter.txt', 'w') as fd:
                print(niter, file=fd)

        g2name, optimizer = task.args
        if optimizer not in data:
            data[optimizer] = {}
        data[optimizer][g2name] = niter


indices = {name: i for i, name in enumerate(g2.names)}

syms = 's o v ^ d p h H X'.split()


print('Optimizer    tests  mean    stdev')
print('--------------------------------')


plt.grid()
for i, opt in enumerate(sorted(data)):

    names = []
    for name in g2.names:
        atoms = g2[name]
        if len(atoms) > 2 and not atoms.has('initial_magmoms'):
            names.append(name)

    y = np.array([data[opt].get(name, -np.inf) for name in names])
    y[y < 0] = -10  #-np.inf
    x = [indices[name] for name in g2.names]

    #y = y.clip(-10, 150)
    good = y[y > 0]
    print('{:15s} {:2d} {:3.5f} {:3.5f}'
          .format(opt, len(good), np.mean(good), np.std(good)))

    #hy, hx = np.histogram(y, bins=20, range=(0, 150))
    #hxavg = (hx[1:] + hx[:-1]) / 2.0
    #plt.plot(hxavg, hy, label=opt, marker='o')
    plt.plot(sorted(y0 for y0 in y if y0 > 0), label=opt)
    plt.axis(ymax=140)

    #plt.scatter(x, np.log(y), marker=syms[i], label=opt)
plt.legend(loc='best')

plt.show()
