import os
from ase.collections import g2
from ase.optimize import BFGS, GPMin
#from ase.optimize.sciopt import ScipyOptimizer as SciOpt
from gpaw import GPAW, Mixer
from gpaw.poisson import FDPoissonSolver
from gpaw.utilities import h2gpts
from gpaw.mpi import world
from contextlib import contextmanager

def optimize(g2name, optimizer, poisson):
    atoms = g2[g2name]
    atoms.center(vacuum=3.0)
    if poisson == 'gs':
        poissonsolver=FDPoissonSolver(relax='GS')
    else:
        poissonsolver=None
    calc = GPAW(mode='lcao', basis='dzp',
                mixer=Mixer(0.5, 7, 50.0),
                convergence=dict(density=1e-5),
                poissonsolver=poissonsolver,
                gpts=h2gpts(0.18, atoms.cell, idiv=8),
                xc='oldLDA', txt='gpaw.txt')

    atoms.rattle(stdev=0.1, seed=1234 + g2.names.index(g2name))
    atoms.calc = calc

    #optcls = BFGS if optimizer == 'bfgs' else GPMin

    kwargs = dict(trajectory='opt.traj',
                  logfile='opt.log')
    if optimizer == 'bfgs':
        opt = BFGS(atoms, maxstep=0.2, **kwargs)
    elif optimizer == 'sciopt':
        opt = SciOpt(atoms, **kwargs)
    else:
        assert optimizer == 'gpmin'
        opt = GPMin(atoms, **kwargs)

    opt.run(fmax=0.01)

workdir = os.getcwd()

for g2name in ['CH3CH2OH']: # g2.names:
    atoms = g2[g2name]
    if len(atoms) == 1 or atoms.has('initial_magmoms'):
        continue

    for poisson in ['fast', 'gs']:
        #optimizer in ['bfgs']:#, 'gpmin']:#, 'sciopt']:
        optimizer = 'bfgs'
        xid = '{}.{}'.format(g2name, poisson)
        dirname = 'out.{}'.format(xid)
        exists = 0
        if world.rank == 0:
            exists = int(os.path.exists(dirname))
            if not exists:
                os.mkdir(dirname)
        exists = world.sum(exists)
        if exists:
            continue
        try:
            os.chdir(dirname)
            optimize(g2name, optimizer, poisson)
        finally:
            os.chdir(workdir)

