import os
from ase.collections import g2
from contextlib import contextmanager

optimizers = ['bfgs', 'bfgs-bad',
              'bfgsls', 'oldqn',
              'fire',
              'lbfgs', 'lbfgsls',
              'gpmin',
              'gpmin-smallstep',
              'sciopt-cg',
              'sciopt-bfgs']


class Task:
    def __init__(self, *args):
        self.args = args

    @property
    def xid(self):
        return '.'.join(self.args)

    @property
    def dirname(self):
        return 'out.{}'.format(self.xid)

    def hasdir(self):
        return os.path.isdir(self.dirname)

    def __str__(self):
        return 'Task({})'.format(self.xid)

    @contextmanager
    def chdir(self):
        pwd = os.getcwd()
        os.chdir(self.dirname)
        yield
        os.chdir(pwd)


def itasks():
    for g2name in g2.names:
        atoms = g2[g2name]
        if len(atoms) < 3 or atoms.has('initial_magmoms'):
            continue

        for optimizer in optimizers:
            yield Task(g2name, optimizer)
